﻿using System;
using System.Collections.Generic;
using System.Text;

namespace ELTE.Basics.University
{
    class Teacher // egyetemi tanár típusa
    {
        private String _Name;
        private String _UniversityCode;
        private List<Course> _Courses;

        public String Name { get { return _Name; } }
        public String UniversityCode { get { return _UniversityCode; } }
        public List<String> CourseNames // a kurzusok neveinek lekérdezése
        {
            get
            {
                List<String> courseNames = new List<String>();
                foreach (Course course in _Courses)
                    courseNames.Add(course.Name); // létrehozunk egy új listát a kurzusokból
                return courseNames;
            }
        }

        public Teacher(String name) // egyetemi tanár létrehozása név alapján
        {
            _Name = name;
            _Courses = new List<Course>();
            _UniversityCode = GenerateCode(_Name); // statikus művelet meghívása

            _ListOfTeachers.Add(this); // statikus mezőhöz hozzáférés
        }

        public Course CreateCourse(String name)
        {
            Course c = new Course(this, name); // nyílt rekurzió használata az oktató objektum átadásával
            _Courses.Add(c);
            return c;
        }
        public Course GetCourse(String name) // kurzus lekérdezése név alapján
        {
            foreach (Course course in _Courses)
                if (course.Name == name) // ha sikerült megtalálnunk
                    return course; // akkor visszaadjuk
            return null; // különben nullát adunk vissza
        }

        // statikus tagok:
        private static List<Teacher> _ListOfTeachers = new List<Teacher>(); 
        // a statikus adatokat vagy a statikus metódusban, vagy helyben inicializáljuk

        public static List<String> ListedTeacherNames // statikus tulajdonság, a kurzusnevek lekérdezésére
        {
            get
            {
                List<String> courseNames = new List<String>();
                foreach (Teacher teacher in _ListOfTeachers)
                    courseNames.Add(teacher.Name); // létrehozunk egy új listát a kurzusokból
                return courseNames;
            }
        }

        private static String GenerateCode(String name) // kódgenerálás, amihez ismerni kell a többi oktató kódját is
        {
            String code = Char.ToUpper(name[0]) + Char.ToUpper(name[1]) + Char.ToUpper(name[name.IndexOf(' ') + 1]) + "IAAT.ELTE"; // kód generálása a név alapján
            Char modifierChar = 'A';
            foreach (Teacher t in _ListOfTeachers)
            {
                if (code == t._UniversityCode) // ha találtunk ugyanolyan nazonosítójú oktatót
                {
                    modifierChar++; // akkor módosítjuk az egyik karaktert
                    code = Char.ToUpper(name[0]) + Char.ToUpper(name[1]) + Char.ToUpper(name[name.IndexOf(' ') + 1]) + "IA" + modifierChar + "T.ELTE";
                }
            }
            return code;
        }
    }
}
